@@ -284,9 +284,11 @@ EVENT_LEVELS = {
 
 DETECTION_TYPES = (
     LIFECYCLE_EVENTS, NODE_STATUS_POLLING, NODE_STATUS_POLL_URL,
+    HYPERVISOR_STATUS_POLLING,
     # LB_STATUS_POLLING,
 ) = (
     'LIFECYCLE_EVENTS', 'NODE_STATUS_POLLING', 'NODE_STATUS_POLL_URL',
+    'HYPERVISOR_STATUS_POLLING',
     # 'LB_STATUS_POLLING',
 )
 
