@@ -25,9 +25,10 @@ from oslo_db import exception as db_exc
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import utils as sa_utils
 from oslo_log import log as logging
+from oslo_utils import importutils
 from oslo_utils import timeutils
-import osprofiler.sqlalchemy
 import sqlalchemy
+from sqlalchemy import and_
 from sqlalchemy.orm import joinedload
 from sqlalchemy.sql.expression import func
 
@@ -37,27 +38,49 @@ from senlin.db.sqlalchemy import migration
 from senlin.db.sqlalchemy import models
 from senlin.db.sqlalchemy import utils
 
+osprofiler_sqlalchemy = importutils.try_import('osprofiler.sqlalchemy')
+
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
-_main_context_manager = None
+_MAIN_CONTEXT_MANAGER = None
 _CONTEXT = threading.local()
 
-cfg.CONF.import_opt('database_retry_limit', 'senlin.conf')
-cfg.CONF.import_opt('database_retry_interval', 'senlin.conf')
-cfg.CONF.import_opt('database_max_retry_interval', 'senlin.conf')
+CONF.import_opt('database_retry_limit', 'senlin.conf')
+CONF.import_opt('database_retry_interval', 'senlin.conf')
+CONF.import_opt('database_max_retry_interval', 'senlin.conf')
+
+try:
+    CONF.import_group('profiler', 'senlin.conf')
+except cfg.NoSuchGroupError:
+    pass
+
+
+def add_db_tracing():
+    global _MAIN_CONTEXT_MANAGER
+
+    if not osprofiler_sqlalchemy:
+        return
+    if not hasattr(CONF, 'profiler'):
+        return
+    if not CONF.profiler.enabled or not CONF.profiler.trace_sqlalchemy:
+        return
+    osprofiler_sqlalchemy.add_tracing(
+        sqlalchemy, _MAIN_CONTEXT_MANAGER.writer.get_engine(), 'db'
+    )
 
 
 def _get_main_context_manager():
-    global _main_context_manager
-    if not _main_context_manager:
-        _main_context_manager = enginefacade.transaction_context()
-        cfg.CONF.import_group('profiler', 'senlin.conf')
-        if cfg.CONF.profiler.enabled:
-            if cfg.CONF.profiler.trace_sqlalchemy:
-                eng = _main_context_manager.writer.get_engine()
-                osprofiler.sqlalchemy.add_tracing(sqlalchemy, eng, "db")
-    return _main_context_manager
+    global _MAIN_CONTEXT_MANAGER
+    if not _MAIN_CONTEXT_MANAGER:
+        _MAIN_CONTEXT_MANAGER = enginefacade.transaction_context()
+        add_db_tracing()
+    return _MAIN_CONTEXT_MANAGER
+
+
+def service_expired_time():
+    return (timeutils.utcnow() -
+            datetime.timedelta(seconds=2.2 * CONF.periodic_interval))
 
 
 def get_engine():
@@ -682,8 +705,8 @@ def cluster_policy_get_all(context, cluster_id, filters=None, sort=None):
             key_name = consts.CP_POLICY_NAME
             if key_type in filters and key_name in filters:
                 query = query.join(models.Policy).filter(
-                    models.Policy.type == filters[key_type] and
-                    models.Policy.name == filters[key_name])
+                    and_(models.Policy.type == filters[key_type],
+                         models.Policy.name == filters[key_name]))
             elif key_type in filters:
                 query = query.join(models.Policy).filter(
                     models.Policy.type == filters[key_type])
@@ -1587,6 +1610,15 @@ def service_get_all():
         return session.query(models.Service).all()
 
 
+def service_get_all_expired(binary):
+    with session_for_read() as session:
+        date_limit = service_expired_time()
+        svc = models.Service
+        return session.query(models.Service).filter(
+            and_(svc.binary == binary, svc.updated_at <= date_limit)
+        )
+
+
 @retry_on_deadlock
 def _mark_engine_failed(session, action_id, timestamp, reason=None):
     query = session.query(models.ActionDependency)
